#import "iBeaconConfigurationViewController.h"
#import "iBeaconDefaults.h"
#import "iBeaconUUIDViewController.h"
#import "iBeaconSignalScene.h"
#import <SpriteKit/SpriteKit.h>

@import CoreLocation;
@import CoreBluetooth;


CBPeripheralManager *peripheralManager = nil;
CLBeaconRegion *region = nil;


@interface iBeaconConfigurationViewController () <CBPeripheralManagerDelegate, UIAlertViewDelegate, UITextFieldDelegate>

@property (nonatomic, weak) IBOutlet UISwitch *enabledSwitch;
@property (nonatomic, weak) IBOutlet UITextField *uuidTextField;

@property (nonatomic, weak) IBOutlet UITextField *majorTextField;
@property (nonatomic, weak) IBOutlet UITextField *minorTextField;
@property (strong, nonatomic) IBOutlet UISlider *powerSlider;
@property (strong, nonatomic) IBOutlet SKView *iBeaconSignalSKView;

@property BOOL enabled;
@property NSUUID *uuid;
@property NSNumber *major;
@property NSNumber *minor;

@property UIBarButtonItem *doneButton;

@property NSNumberFormatter *numberFormatter;

- (void)updateAdvertisedRegion;

@end


@implementation iBeaconConfigurationViewController {
    iBeaconSignalScene *scene;
}
- (IBAction)powerSliderValueChanged:(id)sender {
    [scene stopEmittingSignal];
    [scene startEmittingSignal:(1-_powerSlider.value)];
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    _powerSlider.minimumValue = 0.2;
    _powerSlider.maximumValue = 0.8;
    
    self.doneButton = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemDone target:self action:@selector(doneEditing:)];

    if(region)
    {
        self.uuid = region.proximityUUID;
        self.major = region.major;
        self.minor = region.minor;
    }
    else
    {
        self.uuid = [iBeaconDefaults sharedDefaults].defaultProximityUUID;
        self.major = [NSNumber numberWithShort:0];
        self.minor = [NSNumber numberWithShort:0];
    }
    
    self.numberFormatter = [[NSNumberFormatter alloc] init];
    self.numberFormatter.numberStyle = NSNumberFormatterDecimalStyle;
    
    SKView * skView = (SKView *)self.iBeaconSignalSKView;
    //    skView.showsFPS = YES;
    //    skView.showsNodeCount = YES;
    
    // Create and configure the scene.
    scene = [iBeaconSignalScene sceneWithSize:skView.bounds.size];
    scene.scaleMode = SKSceneScaleModeAspectFill;
    
    // Present the scene.
    [skView presentScene:scene];
}


- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];

    if (!peripheralManager)
    {
        peripheralManager = [[CBPeripheralManager alloc] initWithDelegate:self queue:nil];
    }
    peripheralManager.delegate = self;


    // Refresh the enabled switch.
    self.enabled = self.enabledSwitch.on = peripheralManager.isAdvertising;
    
    self.uuidTextField.text = [self.uuid UUIDString];
    
    self.majorTextField.text = [self.major stringValue];
    self.minorTextField.text = [self.minor stringValue];
}


-(void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];

    peripheralManager.delegate = nil;
}

#pragma mark - Text editing

- (BOOL)textFieldShouldBeginEditing:(UITextField *)textField
{
    if(textField == self.uuidTextField)
    {
        [self performSegueWithIdentifier:@"selectUUID" sender:self];
        return NO;
    }
    
    return YES;
}

- (void)textFieldDidBeginEditing:(UITextField *)textField
{
    self.navigationItem.rightBarButtonItem = self.doneButton;
}


- (void)textFieldDidEndEditing:(UITextField *)textField
{
    if(textField == self.majorTextField)
    {
        self.major = [self.numberFormatter numberFromString:textField.text];
    }
    else if(textField == self.minorTextField)
    {
        self.minor = [self.numberFormatter numberFromString:textField.text];
    }
    
    self.navigationItem.rightBarButtonItem = nil;
    
    [self updateAdvertisedRegion];
}


- (IBAction)toggleEnabled:(UISwitch *)sender
{
    self.enabled = sender.on;
    [self updateAdvertisedRegion];
}


- (IBAction)doneEditing:(id)sender
{
    [self.majorTextField resignFirstResponder];
    [self.minorTextField resignFirstResponder];
    
    [self.tableView reloadData];
}

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender
{
    if([[segue identifier] isEqualToString:@"selectUUID"])
    {
        iBeaconUUIDViewController *uuidSelector = [segue destinationViewController];
        
        uuidSelector.uuid = self.uuid;
    }
}

- (IBAction)unwindUUIDSelector:(UIStoryboardSegue*)sender
{
    iBeaconUUIDViewController *uuidSelector = [sender sourceViewController];
    
    self.uuid = uuidSelector.uuid;
    [self updateAdvertisedRegion];
}

- (void)peripheralManagerDidUpdateState:(CBPeripheralManager *)peripheral
{
    if (peripheral.state == CBPeripheralManagerStatePoweredOn) {
        NSLog(@"iBeaconConfigurationViewController: peripheralManagerDidUpdateState: OK");
    }
}

- (void)peripheralManagerDidStartAdvertising:(CBPeripheralManager *)peripheral error:(NSError *)error
{
    if (error) {
        NSLog(@"iBeaconConfigurationViewController: peripheralManagerDidStartAdvertising: ERROR");
    } else {
        NSLog(@"iBeaconConfigurationViewController: peripheralManagerDidStartAdvertising: OK");
    }
}

- (void)updateAdvertisedRegion
{
    if(peripheralManager.state < CBPeripheralManagerStatePoweredOn)
    {
//        NSString *title = NSLocalizedString(@"Bluetooth must be enabled", @"");
//        NSString *message = NSLocalizedString(@"To configure your device as a beacon", @"");
//        NSString *cancelButtonTitle = NSLocalizedString(@"OK", @"Cancel button title in configuration Save Changes");
//        UIAlertView *errorAlert = [[UIAlertView alloc] initWithTitle:title message:message delegate:self cancelButtonTitle:cancelButtonTitle otherButtonTitles:nil];
//        [errorAlert show];
        
        UIAlertController *errorAlert = [UIAlertController alertControllerWithTitle:@"Error" message:@"Bluetooth must be enabled to configure your device as a beacon." preferredStyle:UIAlertControllerStyleAlert];
        UIAlertAction *okAction = [UIAlertAction actionWithTitle:@"Ok" style:UIAlertActionStyleCancel handler:nil];
        [errorAlert addAction:okAction];
        [self presentViewController:errorAlert animated:YES completion:nil];
        
        return;
    }
    
	[peripheralManager stopAdvertising];
    [scene stopEmittingSignal];
    
    if(self.enabled)
    {
        // We must construct a CLBeaconRegion that represents the payload we want the device to beacon.
        NSDictionary *peripheralData = nil;
        
        region = [[CLBeaconRegion alloc] initWithProximityUUID:self.uuid major:[self.major shortValue] minor:[self.minor shortValue] identifier:BeaconIdentifier];
        peripheralData = [region peripheralDataWithMeasuredPower:nil];
        
        // The region's peripheral data contains the CoreBluetooth-specific data we need to advertise.
        if(peripheralData)
        {
            [peripheralManager startAdvertising:peripheralData];
            [scene startEmittingSignal:_powerSlider.value];
        }
    }
}

@end
