//
//  Messangi.h
//  Messangi
//
//  Created by Francisco Andrades Grassi on 6/2/14.
//  Copyright (c) 2014 Ogangi. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <CoreLocation/CoreLocation.h>
#import "CheckGeoFences.h"
#import "CheckSubscriptions.h"
#import "LoadBeacons.h"

#define MAX_REGIONS  20

#warning OA: GEOFENCELIST_METER_UPDATE_THRESHOLD needs analysis fine-tuning

#define GEOFENCELIST_METER_UPDATE_THRESHOLD 50

@interface Messangi : NSObject <CLLocationManagerDelegate, CheckGeoFencesDelegate, LoadBeaconsDelegate, CheckSubscriptionsDelegate, UITextFieldDelegate>

// ???: i don't know what is stored here
@property (nonatomic, strong) CLLocationManager *messangiLM;
// @property (nonatomic, strong) CLLocationManager *messangiBeaconLM;

/*!
 *  @brief  Last location of device
 */
@property (nonatomic, strong) CLLocation *messangiLastLocation;

/*!
 *  @brief  Set of geoFences after added geoFencesToAdd and removed geoFencesToDelete
 */
@property (nonatomic, strong) NSMutableOrderedSet *geoFencesList;

/*!
 *  @brief  Set of beacons after added beaconsToAdd and removed beaconsToDelete
 */
@property (nonatomic, strong) NSMutableOrderedSet *beaconsList;

/*!
 *  @brief  List of possibles subscriptions
 */
@property (nonatomic, strong) NSMutableArray *subscriptionsList;

/*!
 *  @brief  Name of APP
 */
@property (nonatomic, strong) NSString *appName;

/*!
 *  @brief  Client ID
 */
@property (nonatomic, strong) NSString *clientID;

/*!
 *  @brief  Private Key
 */
@property (nonatomic, strong) NSString *apiClientPrivateKey;

/*!
 *  @brief  Subscription URL
 */
@property (nonatomic, strong) NSString *subscriptionURL;

/*!
 *  @brief  Instance of Subscription
 */
@property (nonatomic, strong) NSString *subscriptionInstanceID;

/*!
 *  @brief Boolean for use Messangi is Activate
 */
@property (nonatomic) BOOL useMessangiSDK;
@property (nonatomic) double lastPendingMessagesFetch;

/*!
 *  @brief  Singleton of Messangi APP
 *  @return Singleton of Messangi
 */
+ (Messangi *)sharedInstance;

/*!
 *  @brief  Init Messangi
 *  @param first A Boolean, if false Messangi manage User Registration, if true the app make your own
 */
- (void)initMessangi:(BOOL)first;

/*!
 *  @brief  <#Description#>
 */
- (void)refreshMonitoredRegionsToClosest;

/*!
 *  @brief Load from subscriptionURL and subscriptionInstanceID, all possible Subscriptions for the app
 */
- (void)initSubscriptions;

/*!
 *  @brief  Subscribe to distribution list
 *  @param subscriptionID An NString use to identify a Distribution List to Subscribe
 */
- (void)subscribeToList:(NSString *)subscriptionID;

/*!
 *  @brief  Unsubscribe from distribution list
 *  @param subscriptionID An NString used to identify a Distribution List to unsubscribe
 */
- (void)unsubscribeFromList:(NSString *)subscriptionID;

/*!
 *  @brief  Every time the app receive a push notification, call this method for process it
 *  @param userInfo          a Dictionary with msgId and type attributes
 *  @param completionHandler ???
 *  @return true if messangi can process the push notification received, false otherwise
 */
- (BOOL)didReceiveRemoteNotification:(NSDictionary *)userInfo fetchCompletionHandler:(void (^)(UIBackgroundFetchResult))completionHandler;


/*!
 *  @brief Internal function called by didReceiveRemoteNotification when arrive a push with type TXT_MSG. Load from Server All Pending Push Messages
 */
- (void)getUnreadMessages;

/*!
 *  @brief Subscribe the device to Messangi Backend
 */
- (void)deviceSubscriberWithUniqueID;

/*!
 *  @brief Compare last deviceToken received with the last saved by the app, if not the same, call deviceSubscriberWithUniqueID
 *  @param deviceToken The deviceToken is provide by apple, and catch by method didRegisterForRemoteNotificationsWithDeviceToken in appDelegate
 */
- (void)registerDeviceToken:(NSData *)deviceToken;

/*!
 *  @brief Start Location Service, from CoreLocation Framework
 */
- (void)startStandardUpdates;

/*!
 *  @brief  Internal function called by didReceiveRemoteNotification when arrive a push with type GET_LOC. Send to Messangi Backend the location of device
 *  @param msgID             A NString with ID
 *  @param completionHandler ???
 */
- (void)sendLastLocationForMsgID:(NSString *)msgID withHandler:(void (^)(UIBackgroundFetchResult))completionHandler;

/*!
 *  @brief  Internal function called by didReceiveRemoteNotification, when arrive a push with type CRT_REGION or UPDT_REGION. Process the msgID NString and create a region with latitude, longitude, radius and trigger and associate with name in set geoFencesToAdd
 *  @param msgID  A NString with this format ID:latitude:longitude:radius:trigger:name
 */
- (void)registerRegionForMsgID:(NSString *)msgID withMessageType:(NSString *)msgType;


/*!
 *  @brief  Internal function called by didReceiveRemoteNotification, when arrive a push with type DLT_REGION. Add to array geoFencesToDelete the msgID
 *  @param msgID A NString with ID
 */
- (void)unregisterRegionWithMsgID:(NSString *)msgID andMessageType:(NSString *)msgType;

/*!
 *  @brief  Internal function called by didReceiveRemoteNotification, when arrive a push with type CRT_BEACON or UPDT_BEACON. Process the msgID NString and create a region with latitude, longitude, radius and trigger and associate with name in set beaconsToAdd
 *  @param msgID  A NString with this format ID:latitude:longitude:radius:trigger:name
 */
- (void)registerBeaconForMsgID:(NSString *)msgID withMessageType:(NSString *)msgType;


/*!
 *  @brief  Internal function called by didReceiveRemoteNotification, when arrive a push with type DLT_BEACON. Add to array beaconsToDelete the msgID
 *  @param msgID A NString with ID
 */
- (void)unregisterBeaconWithMsgID:(NSString *)msgID andMessageType:(NSString *)msgType;


/*!
 *  @brief  Internal function called by didReceiveRemoteNotification, when arrive a push with type GEO_PUSH. Notify message when lateral distance between device location and latitude,longitude are minor than radius
 *  @param msgID             A NString with this format GEO_PUSH:ID:latitude:longitude:radius
 *  @param completionHandler ???
 */
- (void)processGeoPushForMsgID:(NSString *)msgID withHandler:(void (^)(UIBackgroundFetchResult))completionHandler;

/*!
 *  @brief Legacy function not in use at this Messangi Version, was replaced by getUnreadMessages
 *  @param msgID             ???
 *  @param completionHandler ???
 */
- (void)processPushNotificationForMsgID:(NSString *)msgID withHandler:(void (^)(UIBackgroundFetchResult))completionHandler;

/*!
 *  @brief  verify if location service is available
 *  @return YES if is available, otherwise NO
 */
- (BOOL)statusLocationServicesOK;

/*!
 *  @brief  fetch all region monitored by app
 *  @return set of all geoFence Region monitored by app
 */
- (NSSet *)monitoredGeoFences;


/*!
 *  @brief  setter for appName property
 *  @param newName a NSString with app Name
 */
- (void)setAppName:(NSString *)newName;

/*!
 *  @brief  setter for clientID property
 *  @param newClientID a NSString with clientID
 */
- (void)setClientID:(NSString *)newClientID;

/*!
 *  @brief  setter for apiClientPrivateKey property
 *  @param newPrivateKey a NSString with apiClientPrivateKey
 */
- (void)setApiClientPrivateKey:(NSString *)newPrivateKey;

/*!
 *  @brief  setter for subscriptionURL property
 *  @param newURL a NSString with subscriptionURL
 */
- (void)setSubscriptionURL:(NSString *)newURL;

/*!
 *  @brief  setter for subscriptionInstanceID property
 *  @param newID a NSString with subscriptionInstanceID
 */
- (void)setSubscriptionInstanceID:(NSString *)newID;


- (void)tagPendingMessagesFetch;
- (void)untagPendingMessagesFetch;


- (void)logEvent:(NSString *)eventType andExtraInfo:(NSString *)extraInfo;
- (void)logEvent:(NSString *)eventType andExtraInfo:(NSString *)extraInfo withCampaignName: (NSString *) campaignName;

- (void)setLogAllEventsDisabled:(BOOL)disabled;
- (BOOL)isLogAllEventsDisabled;
- (void)setGetLocationDisabled:(BOOL)disabled;
- (BOOL)isGetLocationDisabled;

@end
