//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//:::                                                                         :::
//:::  This routine calculates the distance between two points (given the     :::
//:::  latitude/longitude of those points). It is being used to calculate     :::
//:::  the distance between two locations using GeoDataSource (TM) prodducts  :::
//:::                                                                         :::
//:::  Definitions:                                                           :::
//:::    South latitudes are negative, east longitudes are positive           :::
//:::                                                                         :::
//:::  Passed to function:                                                    :::
//:::    lat1, lon1 = Latitude and Longitude of point 1 (in decimal degrees)  :::
//:::    lat2, lon2 = Latitude and Longitude of point 2 (in decimal degrees)  :::
//:::    unit = the unit you desire for results                               :::
//:::           where: 'M' is statute miles (default)                         :::
//:::                  'K' is kilometers                                      :::
//:::                  'N' is nautical miles                                  :::
//:::                                                                         :::
//:::  Worldwide cities and other features databases with latitude longitude  :::
//:::  are available at http://www.geodatasource.com                          :::
//:::                                                                         :::
//:::  For enquiries, please contact sales@geodatasource.com                  :::
//:::                                                                         :::
//:::  Official Web site: http://www.geodatasource.com                        :::
//:::                                                                         :::
//:::               GeoDataSource.com (C) All Rights Reserved 2015            :::
//:::                                                                         :::
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

function distance(lat1, lon1, lat2, lon2, unit) {
	var radlat1 = Math.PI * lat1/180
	var radlat2 = Math.PI * lat2/180
	var radlon1 = Math.PI * lon1/180
	var radlon2 = Math.PI * lon2/180
	var theta = lon1-lon2
	var radtheta = Math.PI * theta/180
	var dist = Math.sin(radlat1) * Math.sin(radlat2) + Math.cos(radlat1) * Math.cos(radlat2) * Math.cos(radtheta);
	dist = Math.acos(dist)
	dist = dist * 180/Math.PI
	dist = dist * 60 * 1.1515
	if (unit=="K") { dist = dist * 1.609344 }
	if (unit=="N") { dist = dist * 0.8684 }
	return dist
}
'use strict';


var owl;
var geofences = [];

function addNotification(jsonNotify){
  var html = '';
  html +='<li class="card" id="'+jsonNotify.id+'">';
  html +='  <h4 class="title clearfix"><small class="date pull-right">'+moment(jsonNotify.date).format('MMM, D')+'</small><div class="truncate">'+jsonNotify.title+'</div></h4>';
  html +='  <div class="content">';
  html +=     jsonNotify.content;
  html +='  </div>';
  html +='</li>';
  $('#notifications').prepend(html);
}

function updateLocation(jsonLocation){
  $('#latitude').html(jsonLocation.latitude);
  $('#longitude').html(jsonLocation.longitude);
  $('#provider').html(jsonLocation.provider);
  $('#accuracy').html(jsonLocation.accuracy);
}

function updateDistances(jsonLocation){
  var jsonGeofence; 
  for(var g=0 ; g<geofences.length ; g++){
    jsonGeofence = geofences[g];
    $('#'+jsonGeofence.id+' .distance').html(distance(jsonLocation.latitude, jsonLocation.longitude, jsonGeofence.latitude, jsonGeofence.longitude, 'K')*1000);
  }
}

function addGeofence(jsonGeofence){
  geofences.push(jsonGeofence);
  if (geofences.length > 0){
    $('#instrunction1').html('There are '+geofences.length+' geofences.');
    $('#instrunction2').html('Swipe right to view the status');
  }else{
    $('#instrunction1').html('There are no geofences yet.');
    $('#instrunction2').html('Once you receive a geofence it will appear in this section');
  }

  var html = '';
  html += '<div class="card" id="'+jsonGeofence.id+'">';
  html += '  <div class="content text-center">';
  html += '    <p>Geofence type '+jsonGeofence.type+'</p>';
  html += '    <p>The Center is ('+jsonGeofence.latitude+','+jsonGeofence.longitude+')</p>';
  html += '    <p>Radius is '+jsonGeofence.radius+' meters</p>';
  html += '    <p>You are a <span class="distance">UNDEFINED</span> meters from the Center</p>';
  html += '  </div>';
  html += '</div>';

  $('.carousel').slick('slickAdd',html);
}

function removeNotification(id){
  $('#'+id).remove();
}

function shownBeaconSupported(supported){
  var html = '';
  html +='<li class="card">';
  html +='  <h4 class="title">Beacon Support</h4>';
  html +='  <div class="content">';
  html +='    <p>Beacons are '+supported ?'':'NOT'+' supported by this device</p>';
  html +='  </div>';
  html +='</li>';
  $('#notifications').prepend(html);
}

function showVersion(version){
  var html = '';
  html +='<li class="card">';
  html +='  <h4 class="title">Version '+version+'</h4>';
  html +='  <div class="content">';
  html +='  </div>';
  html +='</li>';
  $('#notifications').prepend(html);
}

function init(){
  geofences = [];
  $('.carousel').slick({
    dots: true,
    arrows: false
  });

  var html = '';
  html +='<li class="card">';
  html +='  <h4 class="title">Welcome to Messangi demo app</h4>';
  html +='  <div class="content">';
  html +='    <p>You have not received any push notifications</p>';
  html +='    <p>New messages will be shown in this area. Messages can include rich text and images</p>';
  html +='    <p>Please go to messangi.com to send push notifications</p>';
  html +='  </div>';
  html +='</li>';

  $('#notifications').html(html);
}

$(document).ready(function(){
  init();
  Android.getReady();
});