package com.ogangi.messangi.android.sdk;

import android.app.Activity;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.location.Location;
import android.os.IBinder;

import com.google.android.gms.location.Geofence;
import com.google.android.gms.location.LocationRequest;
import com.ogangi.messangi.android.sdk.vo.MessageVO;
import com.ogangi.messangi.android.sdk.vo.SubscriptionVO;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;


/**
 * Provide the configuration and initialization methods for MessangiSDK
 * also provide the primary methods for interacting with Messangi REST API
 */
@SuppressWarnings("unused")
public class Messangi {
	private static Messangi instance = null;

	private LinkedList<MessangiListener> listeners = null;
	private List<SubscriptionVO> subscriptionList = null;

	private String appName = null;
	private String gcmApiKey = null;
	private String gcmProjectId = null;
	private String apiClientPrivateKey = null;
	private String clientId = null;
	private String subscriptionURL = null;
	private String subscriptionInstanceId = null;

	private boolean useProductionAPI = false;
	private boolean initCompleted = false;
	private Context ctx;
	private Activity activity;

	/**
     *
     * @return String with AppName stored in initialization
     */
	public String getAppName() {
		return appName;
	}

    /**
     *
     * @param appName String with AppName to store in initialization
     */
	public void setAppName(String appName) {
		this.appName = appName;
	}

    /**
     *
     * @return String with the Google Cloud Messaging Api Key stored in initialization
     */
	public String getGcmApiKey() {
		return gcmApiKey;
	}

    /**
     *
     * @param gcmApiKey String with a Google Cloud Messangi APi Key to store in initialization
     */
	public void setGcmApiKey(String gcmApiKey) {
		this.gcmApiKey = gcmApiKey;
	}

    /**
     *
     * @return String with the Google Cloud Messangi Project ID stored in initialization
     */
	public String getGcmProjectId() {
		return gcmProjectId;
	}

    /**
     *
     * @param gcmProjectId String with Google Cloud Messangi Project ID to store in initialization
     */
	public void setGcmProjectId(String gcmProjectId) {
		this.gcmProjectId = gcmProjectId;
	}

    /**
     *
     * @return String with API Client Private Key stored in initialization
     */
	public String getApiClientPrivateKey() {
		return apiClientPrivateKey;
	}

    /**
     *
     * @param apiClientPrivateKey String with API Client Private Key provided by Ogangi
     */
    public void setApiClientPrivateKey(String apiClientPrivateKey) {
        this.apiClientPrivateKey = apiClientPrivateKey;
    }

    /**
     *
     * @return Vector with SubscriptionVO Objects fetches from SubscriptionURL Stored
     */
	public List<SubscriptionVO> getSubscriptionList() {
		return subscriptionList;
	}

    /**
     *
     * @return String with Subscription URL stored in initialization
     */
    public String getSubscriptionURL() {
        return subscriptionURL;
    }

    /**
     *
     * @param subscriptionURL String with Subscription URL to store in initialization
     */
    public void setSubscriptionURL(String subscriptionURL) {
        this.subscriptionURL = subscriptionURL;
    }

    /**
     *
     * @return String with Client ID stored in the initialization
     */
	public String getClientId() {
		return clientId;
	}

    /**
     *
     * @param clientId String with Client ID to store in initialization
     */
	public void setClientId(String clientId) {
		this.clientId = clientId;
	}

    /**
     *
     * @return String with Subscription Instance ID stored in the initialization
     */
	public String getSubscriptionInstanceId() {
		return subscriptionInstanceId;
	}

    /**
     *
     * @param subscriptionInstanceId String with Subscription Instance ID to store in initialization
     */
	public void setSubscriptionInstanceId(String subscriptionInstanceId) {
		this.subscriptionInstanceId = subscriptionInstanceId;
	}

    /**
     * Change Environment used by MessanginSDK
     * @param useProductionAPI true if use Production Environment, false if using staging Environment
     */
    public void setUseProductionAPI(boolean useProductionAPI) {
        this.useProductionAPI = useProductionAPI;

        if (useProductionAPI) {
            Preferences.getInstance(ctx).saveValue(Preferences.REMOTE_API_URL,
                    "https://api.messangi.com/messangi/rest/message/");
        } else {
            Preferences.getInstance(ctx).saveValue(Preferences.REMOTE_API_URL,
                    "https://api.messangi.com/messangi-staging/rest/message/");
        }
    }

    /**
     * Change the messangi endpoint for test Server
     */
    public void useTestingAPI(){
        Preferences.getInstance(ctx).saveValue(Preferences.REMOTE_API_URL,
                "https://api2.messangi.com/messangi-staging/rest/message/");
    }

    /**
     * Check if using Production API
     * @return {code true} if using a production api, false otherwise
     */
    public boolean isUseProductionAPI() {
        return useProductionAPI;
    }

    /**
     *
     * @return List with all MessangiListeners attached to SDK
     */
    public LinkedList<MessangiListener> getListeners() {
        return listeners;
    }

    /*************************************************************************
     *********************** MESSANGI CONFIGURATION **************************
     *************************************************************************/

    /**
     * Constructor for Messangi Class
     */
	private Messangi() {
		listeners = new LinkedList<>();
    }

    /**
     *
     * @return Singleton Instance of Messangi
     */
	public static Messangi getInstance() {
		if (instance == null) {
			instance = new Messangi();
		}
		return instance;
	}

    /**
     * Initialize and start a MessangiService
     * @param context The Context of Activity
     */
    public void init(Context context){
        this.ctx = context;
        Intent intent = new Intent(ctx, MessangiService.class);
        ctx.startService(intent);
    }

    /**
     * Add Listener to LinkedList of Listeners
     * @param listener Instance of MessangiListener to ADD
     */
    public void addMessangiListener(MessangiListener listener) {
        listeners.add(listener);
    }

    /**
     * Remove Listener from LinkedList of Listeners
     * @param listener Instance of MessangiListener to REMOVE
     */
    public void removeMessangiListener(MessangiListener listener) {
        listeners.remove(listener);
    }

    /**
     * Refresh Messangi config parameters with value saved in preferences
     */
	public void reload(){
		if (appName == null) {
			appName = Preferences.getInstance(ctx).getString(Preferences.APP_NAME);
			apiClientPrivateKey = Preferences.getInstance(ctx).getString(Preferences.MESSANGI_API_CLIENT_PRIVATE_KEY);
			clientId = Preferences.getInstance(ctx).getString(Preferences.MESSANGI_API_CLIENT_ID);

			gcmApiKey = Preferences.getInstance(ctx).getString(Preferences.GCM_API_KEY);
			gcmProjectId = Preferences.getInstance(ctx).getString(Preferences.GCM_PROJECT_ID);

			subscriptionURL = Preferences.getInstance(ctx).getString(Preferences.SUBSCRIPTIONS_URL);
			subscriptionInstanceId = Preferences.getInstance(ctx).getString(Preferences.SUBSCRIPTIONS_INSTANCE_ID);
		}
	}

    /**
     * Launch Register Dialog handler by Messangi
     * @param context The app Context
     * @param activity the parent Activity
     */
    public void registerDialog(Context context, Activity activity){
        Preferences pfc = Preferences.getInstance(context);

        if (pfc.getString(Preferences.PHONE_ACTIVATED).trim().equals("") || pfc.getString(Preferences.PHONE_NUMBER).equals("")) {
            new LoginDialog(context, activity).show();
        }
    }

    /**
     * Invoked by init Method, Call postInit method for every MessangiListener Register and Init Location Manager
     */
    public void fireInitCompledEvent() {
        new Thread() {
            public void run() {
                getUnreadMessages(ctx);
                fetchSubscriptionsList(ctx);
                initCompleted = true;
                for (MessangiListener listener : listeners) {
                    try {
                        listener.postInit();
                    } catch (Exception _e) {
                        reportError(_e);
                    }
                }
            }
        }.start();
    }

    /*************************************************************************
     *********************** Messangi Service ********************************
     *************************************************************************/

    /**
     * Boolean value represent when Activity has bind MessangiService
     */
    private boolean mBound;

    /**
     * MessangiService attached to activity
     */
    private MessangiService mService;

    /**
     *
     */
    private ServiceConnection mConnection = new ServiceConnection() {
        @Override
        public void onServiceConnected(ComponentName className, IBinder service) {
            // We've bound to LocalService, cast the IBinder and get LocalService instance
            MessangiService.LocalBinder binder = (MessangiService.LocalBinder) service;
            mService = binder.getService();
            mService.init();
            mBound = true;
        }

        @Override
        public void onServiceDisconnected(ComponentName arg0) {
            mBound = false;
        }
    };

    /**
     * Bind MessangiService to activity
     */
    public void bindService(){
        if(!mBound) {
            Intent intent = new Intent(ctx, MessangiService.class);
            ctx.bindService(intent, mConnection, Context.BIND_ABOVE_CLIENT);
        }
    }

    /**
     * Unbind MessangiService from activity
     */
    public void unBindService(){
        try {
            if (mBound) {
                ctx.unbindService(mConnection);
            }
        }catch (Exception _e){
        }
    }

    /**
     * Get MessangiService bound to activity
     * @return MessangiService bound to activity or null
     */
    public MessangiService getService(){
        if(mBound){
            return mService;
        }else{
            return null;
        }
    }

    //TODO: Agregar un callback en el listener para avisarle que el servicio esta disponible

    /*************************************************************************
     *********************** PUSH NOTIFICATION *******************************
     *************************************************************************/
    /**
     *  Invoked from getUnreadMessages when arrive a Push Notification
     * @param ctx Context of the APP
     * @param message MessageVO with Object of Push Notification Received
     * @param isNew true if The Push Notification is received by first Time, false otherwise
     */
    public void firePushNotificationEvent(Context ctx, MessageVO message, boolean isNew ){
        String fileName = null;
        if (isNew) {
            fileName = Util.storeMessage(ctx,message);
        }

        Util.debug("Firing push notification event to " + listeners.size() + " listeners");

        if (listeners.isEmpty()) {
            if (isNew) {
                Util.storeUndeliveredMessage(ctx, fileName);
            }
        } else {
            for (MessangiListener listener : listeners) {
                try {
                    listener.pushReceived(message);
                } catch (Exception _e) {
                    _e.printStackTrace();
                }
            }
        }

        if (isNew) {
            Intent intent = new Intent("com.ogangi.messangi.android.sdk.PUSH_NOTIFICATION");
            intent.putExtra("message", message.toJson());
            ctx.sendBroadcast(intent);
        }
    }

    /**
     * Method for retrieve all push Notification from server and storage by the app
     * @param context context of the APP
     */
    public void getUnreadMessages(Context context) {
        if (!listeners.isEmpty()) {
            try {
                List<String> undeliveredMessages = Util.getUndeliveredMessages(context);
                for (String name : undeliveredMessages) {
                    System.err.println("Opening: " + name);
                    MessageVO message = Util.getMessage(context, name);
                    if(message != null) {
                        firePushNotificationEvent(context, message, false);
                    }
                }
            } catch (Exception _e) {
                reportError(_e);
            }
        }

        try {
            List<MessageVO> messageList = RemoteAPI.fetchPendingMessages(context);
            if (messageList == null) {
                Util.debug("Connection error");
            } else {
                for(MessageVO message: messageList){
                    Util.debug("Messangi Content:" + message.getMessageTitle() + " ; Content: " + message.getMessageContent());
                    firePushNotificationEvent(context, message, true);
                }
            }
        } catch (Exception _e) {
            reportError(_e);
        }
    }

	/*************************************************************************
	 *********************** LOCATION ****************************************
	 *************************************************************************/

    /**
     * Notify to bound activities when a geofence is triggered
     *
     * @param triggeringGeofences List of geofence triggered
     * @param geofenceTransition The transition triggered
     * @param location the location when trigger notification
     */
    public void fireGeofenceTrigger(List<Geofence> triggeringGeofences, int geofenceTransition, Location location) {
        for (MessangiListener listener : listeners) {
            try {
                listener.updateFencesStatus(triggeringGeofences, geofenceTransition, location);
            } catch (Exception _e) {
                reportError(_e);
            }
        }
    }

    /**
     * Notify to bound activities when messangi detects updates in location
     *
     * @param lastKnowLocation location object with last update location
     */
    public void firelocationUpdate(Location lastKnowLocation){
        for (MessangiListener listener : listeners) {
            try {
                listener.onLocationChange(lastKnowLocation);
            } catch (Exception _e) {
                reportError(_e);
            }
        }
    }

    /**
     * Set custom location type, based in LocationRequest priorities
     * @param type LocationRequest Priority constant
     */
	public void setLocationType(int type){
		Preferences.getInstance(ctx).saveValue(Preferences.LOCATION_UPDATE_TYPE, type);
	}

    /**
     * set default location type available, is LocationRequest.PRIORITY_NO_POWER are default
     */
	public void setLocationType(){
		this.setLocationType(LocationRequest.PRIORITY_NO_POWER);
	}

    /**
     * Set custom time in milliseconds to update location object
     * @param updateTime the update location time in milliseconds
     */
	public void setLocationUpdateTime(long updateTime){
		Preferences.getInstance(ctx).saveValue(Preferences.LOCATION_UPDATE_TIME, String.valueOf(updateTime));
	}

    /**
     * set default time to update location
     */
	public void setLocationUpdateTime(){
		setLocationUpdateTime(10000l); //10secs
	}

    /**
     * Notify to bounds activities when change the list of monitoring geofence
     * @param msgType CRT_REGION, UPDT_REGION, DLT_REGION
     * @param id The id of geofence to add, modify or delete
     */
    public void fireGeofenceUpdate(String msgType, String id) {
        for (MessangiListener listener : listeners) {
            try {
                listener.onGeofenceUpdate(msgType,id);
            } catch (Exception _e) {
                reportError(_e);
            }
        }
    }

    /**
     * Notify to bounds activities when change the list of monitoring beacons
     * @param msgType CRT_BEACON, UPDT_BEACON, DLT_BEACON
     * @param id The id of Beacon to add, modify or delete
     */
    public void fireBeaconUpdate(String msgType, String id) {
        for (MessangiListener listener : listeners) {
            try {
                listener.onBeaconUpdate(msgType, id);
            } catch (Exception _e) {
                reportError(_e);
            }
        }
    }
    /*************************************************************************
     *********************** Subscriptions ***********************************
     *************************************************************************/

    /**
     * Retrieve from the RemoteAPI the subscription list
     * @param context context of the APP
     */
    public List fetchSubscriptionsList(Context context){
        if (getSubscriptionURL() != null) {
            return new ArrayList<SubscriptionVO>(RemoteAPI.getSubscriptions(context));
        }
        return new ArrayList();
    }

    /*************************************************************************
     *********************** LOG & ERROR *************************************
     *************************************************************************/

    /**
     * On error occurred
     * @param _e exception raised
     */
    public void reportError(Exception _e) {
        logEvent(ctx,"Error",_e.getMessage());
        _e.printStackTrace();
    }

    /**
     * Log Custom event without campaign
     * @param context The context of activity
     * @param eventType Event type send to BigData
     * @param extraInfo Info to  associate to eventType in BigData
     */
    public void logEvent(final Context context, final String eventType, final String extraInfo) {
        new Thread() {
            public void run() {
                RemoteAPI.logEvent(context, eventType, extraInfo,null);
            }
        }.start();
    }

    /**
     * Log Custom event with campaign
     * @param context The context of activity
     * @param eventType Event type send to BigData
     * @param extraInfo Info to  associate to eventType in BigData
     * @param campaignName Campaign Name
     */
    public void logEvent(final Context context, final String eventType, final String extraInfo, final String campaignName) {
        new Thread() {
            public void run() {
                RemoteAPI.logEvent(context, eventType, extraInfo,campaignName);
            }
        }.start();
    }
}

