package com.ogangi.messangi.android.sdk;

import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.location.Location;
import android.os.Binder;
import android.os.Environment;
import android.os.IBinder;
import android.os.StrictMode;
import android.telephony.TelephonyManager;
import android.util.Log;

import com.android.gcm.GcmRegister;
import com.google.android.gms.common.ConnectionResult;
import com.google.android.gms.common.GooglePlayServicesUtil;
import com.ogangi.messangi.android.sdk.vo.MessageVO;

import java.io.File;
import java.io.IOException;
import java.util.List;

/**
 * ????
 */
public class MessangiService extends Service {
	// Binder given to clients
	private final IBinder mBinder = new LocalBinder();

    /**
     * When MerssangiService are fully initialize
     */
    private boolean initCompleted;

    private String TAG = "MessangiService";

    /**
	 * Class used for the client Binder.  Because we know this service always
	 * runs in the same process as its clients, we don't need to deal with IPC.
	 */
	public class LocalBinder extends Binder {
		public MessangiService getService() {
			// Return this instance of LocalService so clients can call public methods
			return MessangiService.this;
		}
	}

	@Override
	public IBinder onBind(Intent intent) {
        Log.d(TAG, "onBind");
        Context ctx = getApplicationContext();
        boolean requireMobileNumber = intent.getBooleanExtra("requireMobileNumber",true);

		return mBinder;
	}

    public void init() {
        this.init(false);
    }

    public void init(boolean sdkRegister){
        Preferences pfc = Preferences.getInstance(getApplicationContext());
        if (sdkRegister || !pfc.getString(Preferences.PHONE_ACTIVATED).trim().equals("") && !pfc.getString(Preferences.PHONE_NUMBER).equals("")) {
            Messangi.getInstance().fireInitCompledEvent();
        }
    }

    @Override
    public void onCreate() {
        super.onCreate();
        Util.debug("MessangiService onCreate");
        Log.d(TAG, "onCreate");
        MessangiLocationManager.getInstance().init(getApplicationContext());

        Context ctx = getApplicationContext();
        Preferences pfc = Preferences.getInstance(ctx);

        //Check PushKey and Register
        //TODO: si ya esta registrado omitir este if
        if (checkPlayServices()) {
            // Start IntentService to register this application with GCM.
            Intent intent = new Intent(ctx, GcmRegister.class);
            startService(intent);
        }

        TelephonyManager tm = (TelephonyManager) ctx.getSystemService(Context.TELEPHONY_SERVICE);
        // Saving Phone IMEI
        if (pfc.getString(Preferences.PHONE_IMEI).trim().equals("")) {
            pfc.saveValue(Preferences.PHONE_IMEI, tm.getDeviceId());
            Util.debug("SAVING IMEI: " + pfc.getString(Preferences.PHONE_IMEI));
        } else {
            Util.debug("IMEI: " + pfc.getString(Preferences.PHONE_IMEI));
        }

    }

    @Override
    public void onDestroy() {
        Log.d(TAG, "onDestroy");
        //Last call
        super.onDestroy();
    }

    @Override
	public int onStartCommand(Intent intent, int flags, int startId) {
		Util.debug("MessangiService onStart");
        Log.d(TAG, "onStartCommand");
		return START_STICKY;
	}

	/*************************************************************************
	 *************************** Helper Methods ******************************
	 *************************************************************************/

	/**
	 * Check the device to make sure it has the Google Play Services APK. If
	 * it doesn't, display a dialog that allows users to download the APK from
	 * the Google Play Store or enable it in the device's system settings.
	 */
	private boolean checkPlayServices() {
		int resultCode = GooglePlayServicesUtil.isGooglePlayServicesAvailable(getApplicationContext());
		if (resultCode != ConnectionResult.SUCCESS) {
			if (GooglePlayServicesUtil.isUserRecoverableError(resultCode)) {
				Util.debug(String.valueOf(resultCode));
				//GooglePlayServicesUtil.getErrorDialog(resultCode, activity, PLAY_SERVICES_RESOLUTION_REQUEST).show();
			} else {
				Util.debug("This device is not supported.");
			}
			return false;
		}
		return true;
	}

	/*************************************************************************
	 ************************* Method for Clients ****************************
	 *************************************************************************/


    /*************************************************************************
     *********************** PUSH NOTIFICATION *******************************
     *************************************************************************/

    /**
     * Transform all Messages from Files to MessageVO
     * @param ctx context of the APP
     * @return Array of MessageVO Objects with all Push Received
     */
    public List getAllInboxMessages(Context ctx) {
        return Util.getMessages(ctx);
    }

    /**
     *
     * @param ctx context of the APP
     * @param fileName String with filename where is storing a push notification
     * @return a MessageVO with the info stored in fileName
     */
    public MessageVO loadMessageFromFile(Context ctx, String fileName) {
        return Util.getMessage(ctx, fileName);
    }

    /**
     * Delete push notification stored
     * @param ctx context of the APP
     * @param id The ID of push notification to Delete
     */
    public void deleteMessageFromLocalInbox(Context ctx, String id) {
        Util.deleteMessage(ctx,id);
    }

    /**
     * Delete All storage messages
     * @param ctx
     */
    public void deleteMessages(Context ctx){
        Util.deleteMessages(ctx);
    }

    /*************************************************************************
     *********************** LOCATION ****************************************
     *************************************************************************/

    /**
     * @return List with all Geofence storage
     */
    public List getGeofences(){
        return MessangiLocationManager.getInstance().getGeofenceList();
    }

    /**
     * @return the las know location of the device
     */
    public Location getLastKnownLocation(){
        return MessangiLocationManager.getInstance().getLastKnownLocation();
    }

    /**
     *
     * @return List with all Beacon storage
     */
    public List getBeacons(){
        return MessangiLocationManager.getInstance().getBeaconList();
    }

    /*************************************************************************
     *********************** Subscriptions ***********************************
     *************************************************************************/

    /**
     * Subscribe to distribution list
     * @param context context of the APP
     * @param subId String with the ID of list to Subscribe
     */
    public boolean subscribeToList(Context context, String subId) {
        //Send
        RemoteAPI.subscribeToList(context, subId);
        //Store in local db
        String appName = Preferences.getInstance(getApplicationContext()).getString(Preferences.APP_NAME);
        try {
            final File dir = new File(Environment.getExternalStorageDirectory().getAbsolutePath() + "/" + appName + "/");

            if (!dir.exists() && !dir.mkdirs()) {
                throw new IOException();
            }

            final File myFile = new File(dir, subId + ".txt");

            if(!myFile.createNewFile()){
                Util.debug("File Already Exist");
            }

        } catch (IOException e) {
            e.printStackTrace();
        }
        //TODO: revisar
        return true;
    }

    /**
     * Remove subscription to Distribution List
     * @param context context of the APP
     * @param subId String with the ID of list to Remove subscription
     */
    public boolean unsubscribeFromList(Context context, String subId) {
        RemoteAPI.unsubscribeFromList(context, subId);
        try {
            String appName = Preferences.getInstance(getApplicationContext()).getString(Preferences.APP_NAME);
            final File dir = new File(Environment.getExternalStorageDirectory().getAbsolutePath() + "/" + appName + "/");

            if (!dir.exists() && !dir.mkdirs()) {
                throw new IOException();
            }

            final File myFile = new File(dir, subId + ".txt");

            if (myFile.exists() && !myFile.delete()) {
                throw new IOException();
            }

        } catch (IOException e) {
            e.printStackTrace();
        }
        //TODO: revisar
        return true;
    }

    /**
     * Check if the phone is subscribed to a Distribution List
     * @param subId String with the ID of list to Check
     * @return true if is Subscribed, false otherwise
     */
    public boolean isSaved(String subId) {
        String appName = Preferences.getInstance(getApplicationContext()).getString(Preferences.APP_NAME);
        final File dir = new File(Environment.getExternalStorageDirectory().getAbsolutePath() + "/" + appName + "/");
        if (!dir.exists()) {
            return false;
        }

        final File myFile = new File(dir, subId + ".txt");
        return myFile.exists();
    }

    /*************************************************************************
     *********************** REGISTER & LOGIN ********************************
     *************************************************************************/

    /**
     * In case of you decide make your OWN Registration UI, need call this method for register Phone in Messangi Backend
     * @param ctx context of the APP
     * @param mobile String with Phone Number to Register
     */
    public void registerPhone(Context ctx, String mobile) {

        if (Preferences.getInstance(ctx.getApplicationContext()).getValue(Preferences.PHONE_ACTIVATED).trim()
                .equals("")
                || Preferences.getInstance(ctx).getValue(Preferences.PHONE_NUMBER).equals("")) {

            Preferences.getInstance(ctx).saveValue(Preferences.PHONE_NUMBER, mobile);
            int SDK_INT = android.os.Build.VERSION.SDK_INT;
            if (SDK_INT > 8) {
                StrictMode.ThreadPolicy policy = new StrictMode.ThreadPolicy.Builder().permitAll().build();
                StrictMode.setThreadPolicy(policy);

            }
            RemoteAPI.registerPhone(ctx, mobile);

        }
    }

    /**
     * In case of you decide make your OWN Registration UI, need call this method for validate the SMS Confirmation message received
     * @param ctx context of the APP
     * @param code String with SMS code received by the phone register
     * @return true if validation is correct, false otherwise
     */
    public boolean activatePhone(Context ctx, String code) {
        boolean activationOk;
        if (Preferences.getInstance(ctx.getApplicationContext()).getValue(Preferences.PHONE_ACTIVATED).trim()
                .equals("")
                || Preferences.getInstance(ctx).getValue(Preferences.PHONE_NUMBER).equals("")) {
            int SDK_INT = android.os.Build.VERSION.SDK_INT;
            if (SDK_INT > 8) {
                StrictMode.ThreadPolicy policy = new StrictMode.ThreadPolicy.Builder().permitAll().build();
                StrictMode.setThreadPolicy(policy);

            }
            activationOk = RemoteAPI.activatePhone(ctx, code);
            if (activationOk) {
                Preferences.getInstance(ctx).saveValue(Preferences.ACTIVATION_CODE, code);
                Preferences.getInstance(ctx).saveValue(Preferences.PHONE_ACTIVATED, "true");
            }
        } else {
            activationOk = true;
        }

        return activationOk;
    }

    /**
     * ???
     * @param context context of the APP
     * @param text Text send to Server
     * @param from ???
     * @return ???
     */
    public boolean sendResponse(Context context, String text, String from) {
        return RemoteAPI.sendResponse(context, text, from);
    }

    /*************************************************************************
     *********************** LOG & ERROR *************************************
     *************************************************************************/

    /**
     * On error occurred
     * @param _e exception raised
     */
    public void reportError(Exception _e) {
        logEvent(this,"Error",_e.getMessage());
        _e.printStackTrace();
    }

    /**
     * Log Custom event without campaign
     * @param context The context of activity
     * @param eventType Event type send to BigData
     * @param extraInfo Info to  associate to eventType in BigData
     */
    public void logEvent(final Context context, final String eventType, final String extraInfo) {
        RemoteAPI.logEvent(context, eventType, extraInfo,null);
    }

    /**
     * Log Custom event with campaign
     * @param context The context of activity
     * @param eventType Event type send to BigData
     * @param extraInfo Info to  associate to eventType in BigData
     * @param campaignName Campaign Name
     */
    public void logEvent(final Context context, final String eventType, final String extraInfo, final String campaignName) {
        RemoteAPI.logEvent(context,eventType,extraInfo,campaignName);
    }
}

