package com.ogangi.messangi.android.sdk;

import android.content.Context;
import android.location.Location;

import com.ogangi.messangi.android.sdk.vo.AnswerVO;
import com.ogangi.messangi.android.sdk.vo.BeaconVO;
import com.ogangi.messangi.android.sdk.vo.GeoFenceVO;
import com.ogangi.messangi.android.sdk.vo.MessageVO;
import com.ogangi.messangi.android.sdk.vo.SubscriptionVO;

import org.xmlpull.v1.XmlPullParserException;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.HttpURLConnection;
import java.net.InetAddress;
import java.net.URL;
import java.net.URLEncoder;
import java.net.UnknownHostException;
import java.util.Date;
import java.util.List;
import java.util.Vector;

import javax.net.ssl.HttpsURLConnection;

/**
 * Provide HTTP communication with Messangi REST API
 */
public class RemoteAPI {

    /**
     * Send a GET communication to Messangi REST API and wait the server response
     * @param sUrl a URL to fetch, not need be escaped
     * @return The XML returned by Messangi
     */
    public static String callServer(String sUrl) {
		sUrl = Util.escapeUrl(sUrl);
		Util.debug("callServer with URL: " + sUrl);
		String outputXml = "";
		try {
			URL url = new URL(sUrl);
			HttpsURLConnection conn = (HttpsURLConnection) url.openConnection();
			conn.setUseCaches(false);
			conn.setRequestMethod("GET");
			conn.setAllowUserInteraction(false);
			conn.setRequestProperty("Content-type", "text/xml");
			conn.setRequestProperty("accept", "text/xml");
			BufferedReader in = new BufferedReader(new InputStreamReader(conn.getInputStream()));
			String line;
			while ((line = in.readLine()) != null) {
				outputXml += line;
			}
			in.close();
			conn.disconnect();
			return outputXml;
		} catch (Exception e) {
			e.printStackTrace();
			return "";
		}
	}

    /**
     * Send a GET communication to Messangi REST API and wait the server response
     * @param sUrl URL to fetch
     * @return The XML returned by Messangi
     */
	public static String callServerHTTP(String sUrl) {
		String outputXml = "";
		try {
			URL url = new URL(sUrl);
			HttpURLConnection conn = (HttpURLConnection) url.openConnection();
			conn.setUseCaches(false);
			conn.setRequestMethod("GET");
			conn.setAllowUserInteraction(false);
			conn.setRequestProperty("Content-type", "text/xml");
			conn.setRequestProperty("accept", "text/xml");
			BufferedReader in = new BufferedReader(new InputStreamReader(conn.getInputStream()));
			String line;
			while ((line = in.readLine()) != null) {
				outputXml += line;
			}
			in.close();
			conn.disconnect();
			return outputXml;
		} catch (Exception e) {
			e.printStackTrace();
			return "";
		}
	}

    /**
     * Send a GET communication to Messangi REST API and dont wait the server response
     * @param sUrl URL to fetch
     */
    @SuppressWarnings("unused")
	public static void callServerHTTPWithoutReturn(String sUrl) {
		try {
			URL url = new URL(sUrl);
			HttpURLConnection conn = (HttpURLConnection) url.openConnection();
			conn.setUseCaches(false);
			conn.setRequestMethod("GET");
			conn.setAllowUserInteraction(false);
			BufferedWriter in = new BufferedWriter(new OutputStreamWriter(conn.getOutputStream()));
			in.close();
			conn.disconnect();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

    /**
     * Make Phone Register call to Messangi RESt API
     * @param context context of the APP
     * @param reg_id Register ID send by Google Cloud Messaging
     * @return Object AnswerVO
     */
	public static AnswerVO androidReg(Context context, String reg_id) {
		Preferences prefs = Preferences.getInstance(context);

		try {
			String timeStamp = Util.generateTimeStamp();
			String url_unsigned = prefs.getString(Preferences.PHONE_IMEI) + Constants.SPACER
					+ prefs.getString(Preferences.APP_NAME) + Constants.SPACER + reg_id + Constants.SPACER
					+ prefs.getString(Preferences.MESSANGI_API_CLIENT_ID) + Constants.SPACER + timeStamp;
			String signature = Util.generateSignature(url_unsigned, prefs.getString(Preferences.MESSANGI_API_CLIENT_PRIVATE_KEY));
			String url = prefs.getString(Preferences.REMOTE_API_URL) + Constants.ANDROID_REG + Constants.SPACER
					+ url_unsigned + Constants.SPACER + signature;
			url = Util.escapeUrl(url);
			Util.debug(url);
			try {
                Logger.getInstance(context).addMessage(url,"androidReg-request",new Date().getTime());
				String xmlResponse = callServer(url);
                Logger.getInstance(context).addMessage(xmlResponse,"androidReg-response",new Date().getTime());
				return AnswerVO.parse(xmlResponse);
			} catch (XmlPullParserException e) {
				e.printStackTrace();
				return null;
			} catch (IOException e) {
				e.printStackTrace();
				return null;
			}
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

    /**
     * Register phone number.
     *
     * @param context the context
     * @param number the phone number to register
     */
    public static void registerPhone(Context context, String number) {
		Preferences prefs = Preferences.getInstance(context);
		try {
			String timeStamp = Util.generateTimeStamp();
			String url_unsigned = prefs.getString(Preferences.PHONE_IMEI) + Constants.SPACER
					+ prefs.getString(Preferences.APP_NAME) + Constants.SPACER + Constants.PLATFORM + Constants.SPACER
					+ number + Constants.SPACER + prefs.getString(Preferences.MESSANGI_API_CLIENT_ID) + Constants.SPACER + timeStamp;
			String signature = Util.generateSignature(url_unsigned, prefs.getString(Preferences.MESSANGI_API_CLIENT_PRIVATE_KEY));
			String url = prefs.getString(Preferences.REMOTE_API_URL) + Constants.REGISTER_PHONE + Constants.SPACER
					+ url_unsigned + Constants.SPACER
					+ signature;
			url = Util.escapeUrl(url);
			Util.debug(url);

            Logger.getInstance(context).addMessage(url, "registerPhone-request", new Date().getTime());
            String xmlResponse = callServer(url);
            Logger.getInstance(context).addMessage(xmlResponse, "registerPhone-response", new Date().getTime());
			if(xmlResponse.length() == 0){
				throw new NullPointerException();
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

    /**
     * Validate phone. After registration arrive a SMS code you need
     *
     * @param context the context
     * @param activationCode the activation code send by SMS
     * @return <code>true</code> if is a Valid Code, <code>false</code> otherwise
     */
    public static boolean activatePhone(Context context, String activationCode) {
		Preferences prefs = Preferences.getInstance(context);
		try {
			String timeStamp = Util.generateTimeStamp();
			String url_unsigned = prefs.getString(Preferences.PHONE_IMEI) + Constants.SPACER
					+ prefs.getString(Preferences.APP_NAME) + Constants.SPACER + Constants.PLATFORM + Constants.SPACER
					+ activationCode + Constants.SPACER + prefs.getString(Preferences.MESSANGI_API_CLIENT_ID) + Constants.SPACER
					+ timeStamp;
			String signature = Util.generateSignature(url_unsigned, prefs.getString(Preferences.MESSANGI_API_CLIENT_PRIVATE_KEY));
			String url = prefs.getString(Preferences.REMOTE_API_URL) + Constants.ACTIVATE_PHONE + Constants.SPACER
					+ url_unsigned + Constants.SPACER
					+ signature;
			url = Util.escapeUrl(url);
			Util.debug(url);
            try {
                Logger.getInstance(context).addMessage(url, "activatePhone-request", new Date().getTime());
                String xmlResponse = callServer(url);
                Logger.getInstance(context).addMessage(xmlResponse, "activatePhone-response", new Date().getTime());
                AnswerVO tmp = AnswerVO.parse(xmlResponse);
                return tmp.getStatus().equalsIgnoreCase("true");
            } catch (XmlPullParserException e) {
                e.printStackTrace();
                return false;
            } catch (IOException e) {
                e.printStackTrace();
                return false;
            }
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		}
	}

    /**
     * Fetch message from server.
     *
     * @param context the context
     * @param message_id the Id of Message to retrieve
     * @return The Message cast in MessageVO
     */
    public static MessageVO fetchMessage(Context context, String message_id) {
		Preferences prefs = Preferences.getInstance(context);
		try {
			String timeStamp = Util.generateTimeStamp();
			String url_unsigned = message_id + Constants.SPACER + prefs.getString(Preferences.MESSANGI_API_CLIENT_ID)
					+ Constants.SPACER + timeStamp;
			String signature = Util.generateSignature(url_unsigned, prefs.getString(Preferences.MESSANGI_API_CLIENT_PRIVATE_KEY));
			String url = prefs.getString(Preferences.REMOTE_API_URL) + Constants.MESSAGE_CONTENT + Constants.SPACER
					+ url_unsigned + Constants.SPACER + signature;
			url = Util.escapeUrl(url);
			Util.debug(url);
            try {
                Logger.getInstance(context).addMessage(url, "fetchMessage-request", new Date().getTime());
                String xmlResponse = callServer(url);
                Logger.getInstance(context).addMessage(xmlResponse, "fetchMessage-response", new Date().getTime());
                return MessageVO.parse(xmlResponse);
            } catch (XmlPullParserException e) {
                e.printStackTrace();
                return null;
            } catch (IOException e) {
                e.printStackTrace();
                return null;
            }

		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

    /**
     * Fetch all pending messages.
     *
     * @param context the context
     * @return A Vector with all Message retrieve from server
     */
    public static Vector<MessageVO> fetchPendingMessages(Context context) {
		Preferences prefs = Preferences.getInstance(context);
        
		try {
			String timeStamp = Util.generateTimeStamp();
			String url_unsigned = prefs.getString(Preferences.PHONE_IMEI) + Constants.SPACER + prefs.getString(Preferences.APP_NAME)
					+ Constants.SPACER + Constants.PLATFORM + Constants.SPACER + prefs.getString(Preferences.MESSANGI_API_CLIENT_ID)
					+ Constants.SPACER + timeStamp;
			String signature = Util.generateSignature(url_unsigned, prefs.getString(Preferences.MESSANGI_API_CLIENT_PRIVATE_KEY));
			String url = prefs.getString(Preferences.REMOTE_API_URL) + Constants.PENDING_MESSAGES + Constants.SPACER
					+ url_unsigned + Constants.SPACER + signature;
			url = Util.escapeUrl(url);
			Util.debug(url);

            try {
                Logger.getInstance(context).addMessage(url, "fetchPendingMessages-request", new Date().getTime());
                String xmlResponse = callServer(url);
                Logger.getInstance(context).addMessage(xmlResponse, "fetchPendingMessages-response", new Date().getTime());
                return new Vector<MessageVO>(MessageVO.parseMultiple(xmlResponse));
            } catch (XmlPullParserException e) {
                e.printStackTrace();
                return null;
            } catch (IOException e) {
                e.printStackTrace();
                return null;
            }


		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

    /**
     * Gets all subscriptions possibles.
     *
     * @param context the context
     * @return the subscriptions
     */
    public static List getSubscriptions(Context context) {
        Preferences prefs = Preferences.getInstance(context);

		try {
            //https://www.messangi.com/messangi_mmc/rest/api/distributionlists/8/a
			String url = prefs.getString(Preferences.SUBSCRIPTIONS_URL) + Constants.SPACER + Constants.GET_SUBSCRIPTIONS
					+ Constants.SPACER + prefs.getString(Preferences.SUBSCRIPTIONS_INSTANCE_ID) + Constants.SPACER + "a";
			Util.debug(url);
            try {
                Logger.getInstance(context).addMessage(url, "getSubscriptions-request", new Date().getTime());
                String xmlResponse = callServer(url);
                Logger.getInstance(context).addMessage(xmlResponse, "getSubscriptions-response", new Date().getTime());
                return SubscriptionVO.parses(xmlResponse);
            } catch (XmlPullParserException e) {
                e.printStackTrace();
                return null;
            } catch (IOException e) {
                e.printStackTrace();
                return null;
            }
		} catch (Exception e) {
			e.printStackTrace();
			Util.debug("sub_" + e.getClass().toString());
			return null;
		}
	}

    /**
     * Subscribe to list.
     *
     * @param context the context
     * @param subId the ID of subscription list to subscribe
     */
    public static void subscribeToList(Context context, String subId) {
		Preferences prefs = Preferences.getInstance(context);
        //<subscriptionURL>/addtodistlist/<subscriptionID>/<mobileNumber>
        String url = prefs.getString(Preferences.SUBSCRIPTIONS_URL) + Constants.SPACER + Constants.SUBSCRIBE_TO_LIST
                + Constants.SPACER + subId + Constants.SPACER + prefs.getString(Preferences.PHONE_NUMBER);
        Util.debug(url);
		try {
            Logger.getInstance(context).addMessage(url, "subscribeToList-request", new Date().getTime());
            String xmlResponse = callServer(url);
            Logger.getInstance(context).addMessage(xmlResponse, "subscribeToList-response", new Date().getTime());
            //TODO: Procesar la xmlRespose
		} catch (Exception e) {
			e.printStackTrace();
			Util.debug("sub_" + e.getClass().toString());
		}
	}

    /**
     * Unsubscribe from list.
     *
     * @param context the context
     * @param subId The ID of subscription list to Unsubscribe
     */
    public static void unsubscribeFromList(Context context, String subId) {
		Preferences prefs = Preferences.getInstance(context);
		try {
			String url = prefs.getString(Preferences.SUBSCRIPTIONS_URL) + Constants.SPACER
					+ Constants.UNSUBSCRIBE_FROM_LIST + Constants.SPACER + subId + Constants.SPACER
					+ prefs.getString(Preferences.PHONE_NUMBER);
			Util.debug(url);
            Logger.getInstance(context).addMessage(url, "unsubscribeFromList-request", new Date().getTime());
            String xmlResponse = callServer(url);
            Logger.getInstance(context).addMessage(xmlResponse, "unsubscribeFromList-response", new Date().getTime());
            //TODO: Procesar la xmlRespose
		} catch (Exception e) {
			e.printStackTrace();
			Util.debug("sub_" + e.getClass().toString());
		}
	}

    /**
     * Gets geoFences.
     *
     * @param context the context
     */
    @SuppressWarnings("unchecked")
	public static void getGeofences(Context context) {
		Preferences prefs = Preferences.getInstance(context);
		try {
			String timeStamp = Util.generateTimeStamp();
			String url_unsigned = prefs.getString(Preferences.APP_NAME) + Constants.SPACER
					+ prefs.getString(Preferences.MESSANGI_API_CLIENT_ID) + Constants.SPACER + timeStamp;
			String signature = Util.generateSignature(url_unsigned, prefs.getString(Preferences.MESSANGI_API_CLIENT_PRIVATE_KEY));
			String url = prefs.getString(Preferences.REMOTE_API_URL) + Constants.GEOFENCES_LIST + Constants.SPACER
					+ url_unsigned + Constants.SPACER + signature;
			url = Util.escapeUrl(url);
			Util.debug(url);

            try {
                Logger.getInstance(context).addMessage(url, "getGeofences-request", new Date().getTime());
                String xmlResponse = callServer(url);
                Logger.getInstance(context).addMessage(xmlResponse, "getGeofences-response", new Date().getTime());
                List<GeoFenceVO> geofences = GeoFenceVO.parse(xmlResponse);
                for (GeoFenceVO geoFence : geofences) {
					MessangiLocationManager.getInstance().addGeofence(geoFence);
                }
            }catch (XmlPullParserException|IOException e) {
                e.printStackTrace();
            }
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

    /**
     * Gets beacons.
     *
     * @param context the context
     */
    @SuppressWarnings("unchecked")
	public static void getBeacons(Context context) {
		Preferences prefs = Preferences.getInstance(context);
		try {
			String timeStamp = Util.generateTimeStamp();
			String url_unsigned = prefs.getString(Preferences.APP_NAME) + Constants.SPACER
					+ prefs.getString(Preferences.MESSANGI_API_CLIENT_ID) + Constants.SPACER + timeStamp;
			String signature = Util.generateSignature(url_unsigned, prefs.getString(Preferences.MESSANGI_API_CLIENT_PRIVATE_KEY));
			String url = prefs.getString(Preferences.REMOTE_API_URL) + Constants.BEACONS_LIST + Constants.SPACER
					+ url_unsigned + Constants.SPACER + signature;
			url = Util.escapeUrl(url);
			Util.debug(url);

            try {
                Logger.getInstance(context).addMessage(url, "getBeacons-request", new Date().getTime());
                String xmlResponse = callServer(url);
                Logger.getInstance(context).addMessage(xmlResponse, "getBeacons-response", new Date().getTime());
                List<BeaconVO> beacons = BeaconVO.parse(xmlResponse);
                MessangiLocationManager.getInstance().removeAllBeacons();
                for (BeaconVO beacon : beacons) {
                    // 2472f171-d26f-45d4-bff6-1df0f1d9b5e5:B9407F30-F5F8-466E-AFF9-25556B57FE6D:1010:9090:ESTIMOTE:IMMEDIATE:ENTER:Fruits
                    MessangiLocationManager.getInstance().addBeacon(beacon.getRegionId(), beacon.toString());
                }
            }catch (XmlPullParserException|IOException e) {
                e.printStackTrace();
            }
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

    /**
     * Geo fence event.
     *
     * @param context the context
     * @param fence_id the FenceId
     * @param status the Status
     * @param lat the latitude
     * @param lon the longitude
     * @param type the type
     */
    public static void geoFenceEvent(Context context, String fence_id, String status, double lat, double lon, String type) {
		Preferences prefs = Preferences.getInstance(context);
		try {
			String timeStamp = Util.generateTimeStamp();
			String url_unsigned = fence_id + Constants.SPACER + prefs.getString(Preferences.PHONE_IMEI)
					+ Constants.SPACER + status + Constants.SPACER + timeStamp + Constants.SPACER + lat
					+ Constants.SPACER + lon + Constants.SPACER + type + Constants.SPACER
					+ prefs.getString(Preferences.APP_NAME) + Constants.SPACER + Constants.PLATFORM + Constants.SPACER
					+ prefs.getString(Preferences.MESSANGI_API_CLIENT_ID) + Constants.SPACER + timeStamp;
			String signature = Util.generateSignature(url_unsigned, prefs.getString(Preferences.MESSANGI_API_CLIENT_PRIVATE_KEY));
			String url = prefs.getString(Preferences.REMOTE_API_URL) + Constants.REGION_EVENT + Constants.SPACER
					+ url_unsigned + Constants.SPACER + signature;
			url = Util.escapeUrl(url);
			Util.debug(url);
			try {
                Logger.getInstance(context).addMessage(url, "geoFenceEvent-request", new Date().getTime());
                String xmlResponse = callServer(url);
                Logger.getInstance(context).addMessage(xmlResponse, "geoFenceEvent-response", new Date().getTime());
                //TODO: Parsear correctamente la respuesta del servidor
			} catch (Exception e) {
				e.printStackTrace();
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

    /**
     * Send location.
     *
     * @param context the context
     * @param requestId the request id
     * @param lat the latitude
     * @param lon the longitude
     */
    public static void sendLocation(Context context, String requestId, double lat, double lon) {
		Preferences prefs = Preferences.getInstance(context);
		try {
			String timeStamp = Util.generateTimeStamp();
			String url_unsigned = requestId + Constants.SPACER + prefs.getString(Preferences.PHONE_NUMBER)
					+ Constants.SPACER + timeStamp + Constants.SPACER + lat + Constants.SPACER + lon + Constants.SPACER
					+ "APP" + Constants.SPACER + prefs.getString(Preferences.MESSANGI_API_CLIENT_ID) + Constants.SPACER + timeStamp;
			String signature = Util.generateSignature(url_unsigned, prefs.getString(Preferences.MESSANGI_API_CLIENT_PRIVATE_KEY));
			String url = prefs.getString(Preferences.REMOTE_API_URL) + Constants.GEOLOCATION_EVENT + Constants.SPACER
					+ url_unsigned
					+ Constants.SPACER + signature;
			url = Util.escapeUrl(url);
			Util.debug(url);
            try {
                Logger.getInstance(context).addMessage(url, "sendLocation-request", new Date().getTime());
                String xmlResponse = callServer(url);
                Logger.getInstance(context).addMessage(xmlResponse, "sendLocation-response", new Date().getTime());
                //TODO: Parsear correctamente la respuesta del servidor
            } catch (Exception e) {
                e.printStackTrace();
            }
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

    /**
     * Update message.
     *
     * @param context the context
     * @param message_id the message _ id
     * @param status the status
     * @return the boolean
     */
    @SuppressWarnings("unused")
    public static boolean updateMessage(Context context, String message_id, String status) {
		Preferences prefs = Preferences.getInstance(context);
		try {
			String timeStamp = Util.generateTimeStamp();
			String url_unsigned = message_id + Constants.SPACER + status + Constants.SPACER
					+ prefs.getString(Preferences.MESSANGI_API_CLIENT_ID) + Constants.SPACER + timeStamp;
			String signature = Util.generateSignature(url_unsigned, prefs.getString(Preferences.MESSANGI_API_CLIENT_PRIVATE_KEY));
			String url = prefs.getString(Preferences.REMOTE_API_URL) + Constants.UPDATE_MESSAGE_STATUS
					+ Constants.SPACER + url_unsigned + Constants.SPACER
					+ signature;
			url = Util.escapeUrl(url);
            Util.debug(url);
            try {
                Logger.getInstance(context).addMessage(url, "updateMessage-request", new Date().getTime());
                String xmlResponse = callServer(url);
                Logger.getInstance(context).addMessage(xmlResponse, "updateMessage-response", new Date().getTime());
                AnswerVO tmp = AnswerVO.parse(xmlResponse);
                return tmp.getStatus().equalsIgnoreCase("true");
            } catch (XmlPullParserException e) {
                e.printStackTrace();
                return false;
            } catch (IOException e) {
                e.printStackTrace();
                return false;
            }
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		}
	}

    /**
     * ????  Send response.
     *
     * @param context the context
     * @param text the text
     * @param from the from
     * @return the boolean
     */
    public static boolean sendResponse(Context context, String text, String from) {
		Preferences prefs = Preferences.getInstance(context);
		try {
			text = new String(Util.safeBase64Encode(text));
			String timeStamp = Util.generateTimeStamp();
			String url_unsigned = text + Constants.SPACER + from + Constants.SPACER
					+ prefs.getString(Preferences.APP_NAME) + Constants.SPACER + Constants.PLATFORM + Constants.SPACER
					+ prefs.getString(Preferences.MESSANGI_API_CLIENT_ID) + Constants.SPACER + timeStamp;
			String signature = Util.generateSignature(url_unsigned, prefs.getString(Preferences.MESSANGI_API_CLIENT_PRIVATE_KEY));
			String url = prefs.getString(Preferences.REMOTE_API_URL) + Constants.SEND_RESPONSE + Constants.SPACER
					+ url_unsigned + Constants.SPACER + signature;
			url = Util.escapeUrl(url);
            Util.debug(url);
            try {
                Logger.getInstance(context).addMessage(url, "sendResponse-request", new Date().getTime());
                String xmlResponse = callServer(url);
                Logger.getInstance(context).addMessage(xmlResponse, "sendResponse-response", new Date().getTime());
                AnswerVO tmp = AnswerVO.parse(xmlResponse);
                return tmp.getStatus().equalsIgnoreCase("true");
            } catch (XmlPullParserException e) {
                e.printStackTrace();
                return false;
            } catch (IOException e) {
                e.printStackTrace();
                return false;
            }
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		}
	}

	/**
	 * Register Event in Ogangi BigData Platform
	 * @param context The app Contex
	 * @param eventType Event Type to Register
	 * @param extraInfo Relevant Information to register
	 */
	public static void logEvent(Context context, String eventType, String extraInfo, String campaignName){
		Preferences prefs = Preferences.getInstance(context);
		String timeStamp = Util.generateTimeStamp();


		String url = "https://api.messangi.com/events/"
						+"?deviceid="+prefs.getString(Preferences.PHONE_IMEI).trim()
						+"&client=" + prefs.getString(Preferences.MESSANGI_API_CLIENT_ID)
						+"&app=" + prefs.getString(Preferences.APP_NAME)
						+"&platform=Android&eventtype="+eventType
						+"&timestamp="+timeStamp;

		try {
			Location location = MessangiLocationManager.getInstance().getLastKnownLocation();
			if (location != null) {
				url += "&latitude=" + location.getLatitude() + "&longitude=" + location.getLongitude();
			}
		}catch (Exception _e){
			_e.printStackTrace();
		}

		try {
			String mobile = Preferences.getInstance(context).getString(Preferences.PHONE_NUMBER);
			if (mobile != null) {
				url += "&mobile=" + URLEncoder.encode(mobile.trim(), "UTF-8");
			}
		}catch (Exception _e){
			_e.printStackTrace();
		}

		try {
			if (extraInfo != null) {
				url += "&extrainfo=" + URLEncoder.encode(extraInfo, "UTF-8");
			}
		}catch (Exception _e){
			_e.printStackTrace();
		}

		try {
			String versionName = context.getPackageManager().getPackageInfo(context.getPackageName(), 0).versionName;
			if (versionName != null) {
				url += "&version=" + URLEncoder.encode(versionName, "UTF-8");
			}
		}catch (Exception _e){
			url += "&version=2E";
			_e.printStackTrace();
		}

		try{
			if (campaignName != null){
				url += "&campaign_name="+ URLEncoder.encode(campaignName,"UTF-8");
			}
		}catch (Exception _e){
			_e.printStackTrace();
		}

		url = Util.escapeUrl(url);
		Util.debug(url);

		try {
			if(Preferences.getInstance(context).getBoolean(Preferences.LOGEVENTS_ALLOWED,true)) {
				String response = callServer(url);
				Util.debug("Event: '" + response + "'");
			}
		} catch (Exception _e) {
			_e.printStackTrace();
		}
	}

	/**
	 * Returns <code>true</code> if Messangi Backend is Reached
	 *
	 * @return <code>true</code> if Messangi Backend is Reached
	 *         <code>false</code> otherwise
	 */
	public static boolean checkConnection(Context context){
		try {
			InetAddress ipAddr = InetAddress.getByName("api.messangi.com");
			return !ipAddr.equals("");
		} catch (UnknownHostException e) {
            e.printStackTrace();
            return false;
        }
    }
}

