package com.ogangi.messangi.android.sdk.vo;

import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.util.Xml;

import com.google.android.gms.location.Geofence;

import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;

import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

/**
 * Represents a Geofence Object retrieved from Messangi
 */
public class GeoFenceVO {
    private static final String geoEnterTrigger = "ENTER";
    private static final String geoExitTrigger = "EXIT";
    private static final String geoBothTrigger = "ENTER-EXIT";

    // Instance variables
    private final String id;
    private final double latitude;
    private final double longitude;
    private final float radius;
    private String eventType;
    private String name;

    private long expirationDuration;
    private int transitionType;

    private long creationTime;
    private long lastEnterTime;
    private long lastExitTime;


    public GeoFenceVO(String id, String latitude, String longitude, String radius, String eventType, String name) {
        this.id = id;
        this.latitude = Double.parseDouble(latitude);
        this.longitude = Double.parseDouble(longitude);
        this.radius = Float.parseFloat(radius);
        this.eventType = eventType;
        this.name = name;

        switch (eventType) {
            case geoEnterTrigger:
                this.transitionType = Geofence.GEOFENCE_TRANSITION_ENTER;
                break;
            case geoExitTrigger:
                this.transitionType = Geofence.GEOFENCE_TRANSITION_EXIT;
                break;
            default:
                this.transitionType = Geofence.GEOFENCE_TRANSITION_ENTER | Geofence.GEOFENCE_TRANSITION_EXIT;
        }
        this.creationTime = Calendar.getInstance().getTimeInMillis();
        this.lastEnterTime = -1;
        this.lastExitTime = -1;
    }

    public GeoFenceVO(String id, double latitude, double longitude, float radius, String name, long expirationDuration, int transitionType) {
        this.id = id;
        this.latitude = latitude;
        this.longitude = longitude;
        this.radius = radius;
        this.expirationDuration = expirationDuration;
        this.transitionType = transitionType;
        this.creationTime = Calendar.getInstance().getTimeInMillis();
        this.lastEnterTime = -1;
        this.lastExitTime = -1;
        this.name = name;

        switch (transitionType) {
            case Geofence.GEOFENCE_TRANSITION_ENTER:
                this.eventType = geoEnterTrigger;
                break;
            case Geofence.GEOFENCE_TRANSITION_EXIT:
                this.eventType = geoExitTrigger;
                break;
            default:
                this.eventType = geoBothTrigger;
        }

    }

    public GeoFenceVO(String id, double latitude, double longitude, float radius, String name, long expirationDuration, int transitionType, long creationTime, long lastEnterTime, long lastExitTime) {
        this.id = id;
        this.latitude = latitude;
        this.longitude = longitude;
        this.radius = radius;
        this.expirationDuration = expirationDuration;
        this.transitionType = transitionType;
        this.creationTime = creationTime;
        this.lastEnterTime = lastEnterTime;
        this.lastExitTime = lastExitTime;
        this.name = name;

        switch (transitionType) {
            case Geofence.GEOFENCE_TRANSITION_ENTER:
                this.eventType = geoEnterTrigger;
                break;
            case Geofence.GEOFENCE_TRANSITION_EXIT:
                this.eventType = geoExitTrigger;
                break;
            default:
                this.eventType = geoBothTrigger;
        }
    }

    public String getId() {
        return id;
    }

    public double getLatitude() {
        return latitude;
    }

    public double getLongitude() {
        return longitude;
    }

    public float getRadius() {
        return radius;
    }

    public String getEventType() {
        return eventType;
    }

    public void setEventType(String eventType) {
        this.eventType = eventType;
        switch (eventType) {
            case geoEnterTrigger:
                this.transitionType = Geofence.GEOFENCE_TRANSITION_ENTER;
                break;
            case geoExitTrigger:
                this.transitionType = Geofence.GEOFENCE_TRANSITION_EXIT;
                break;
            default:
                this.transitionType = Geofence.GEOFENCE_TRANSITION_ENTER | Geofence.GEOFENCE_TRANSITION_EXIT;
        }
    }

    public String getName() {
        if(name != null)
            return name;
        else
            return "";
    }

    public void setName(String name) {
        this.name = name;
    }

    public long getExpirationDuration() {
        return expirationDuration;
    }

    public void setExpirationDuration(long expirationDuration) {
        this.expirationDuration = expirationDuration;
    }

    public int getTransitionType() {
        return transitionType;
    }

    public void setTransitionType(int transitionType) {
        this.transitionType = transitionType;
        switch (transitionType) {
            case Geofence.GEOFENCE_TRANSITION_ENTER:
                this.eventType = geoEnterTrigger;
                break;
            case Geofence.GEOFENCE_TRANSITION_EXIT:
                this.eventType = geoExitTrigger;
                break;
            default:
                this.eventType = geoBothTrigger;
        }
    }

    public long getCreationTime() {
        return creationTime;
    }

    public void setCreationTime(long creationTime) {
        this.creationTime = creationTime;
    }

    public long getLastExitTime() {
        return lastExitTime;
    }

    public void setLastExitTime(long lastExitTime) {
        this.lastExitTime = lastExitTime;
    }

    public long getLastEnterTime() {
        return lastEnterTime;
    }

    public void setLastEnterTime(long lastEnterTime) {
        this.lastEnterTime = lastEnterTime;
    }

    /**
     * Creates a Location Services Geofence object from a SimpleGeofence.
     * @return A Geofence object.
     */
    @NonNull
    public Geofence toGeofence() {
        // Build a new Geofence object.
        return new Geofence.Builder()
                .setRequestId(id)
                .setTransitionTypes(transitionType)
                .setCircularRegion(latitude, longitude, radius)
                .setExpirationDuration(Geofence.NEVER_EXPIRE)
                .build();
    }

    @NonNull
    public String toString(){
        String message;
        message = this.id;
        message += ":" + this.latitude;
        message += ":" + this.longitude;
        message += ":" + this.radius;
        message += ":" + this.eventType;
        message += ":" + this.name;
        return message;
    }

    //TODO: Considerar mover esto a un padre comun de todas las clases VO y extender
    private static final String ns = null;

    @NonNull
    public static List<GeoFenceVO> parse(String in) throws XmlPullParserException, IOException {
        XmlPullParser parser = Xml.newPullParser();
        parser.setFeature(XmlPullParser.FEATURE_PROCESS_NAMESPACES, false);
        parser.setInput(new StringReader(in));
        parser.nextTag();
        return readFeed(parser);
    }

    @NonNull
    private static List<GeoFenceVO> readFeed(XmlPullParser parser) throws XmlPullParserException, IOException {
        List<GeoFenceVO> entries = new ArrayList<>();

        parser.require(XmlPullParser.START_TAG, ns, "geoFences");
        while (parser.next() != XmlPullParser.END_TAG) {
            if (parser.getEventType() != XmlPullParser.START_TAG) {
                continue;
            }
            String name = parser.getName();
            // Starts by looking for the entry tag
            if (name.equals("geoFence")) {
                GeoFenceVO geofence = parseGeoFenceVO(parser);
                if( null != geofence) {
                    entries.add(geofence);
                }
            } else {
                skip(parser);
            }
        }
        return entries;
    }

    // Parses the contents of an XML entry. If it encounters a successful or message tag, hands them off
    // to their respective "read" methods for processing. Otherwise, skips the tag.
    @Nullable
    public static GeoFenceVO parseGeoFenceVO(XmlPullParser parser) throws XmlPullParserException, IOException {
        parser.require(XmlPullParser.START_TAG, ns, "geoFence");

        String geoFenceId = null;
        String latitude = null;
        String longitude = null;
        String radius = null;
        String eventType = null;
        String name = null;
        String activated = null;

        while (parser.next() != XmlPullParser.END_TAG) {
            if (parser.getEventType() != XmlPullParser.START_TAG) {
                continue;
            }
            String nameTag = parser.getName();

            switch (nameTag){
                case "geoFenceId":
                    geoFenceId = read(parser,nameTag);
                    break;
                case "latitude":
                    latitude = read(parser,nameTag);
                    break;
                case "longitude":
                    longitude = read(parser,nameTag);
                    break;
                case "radius":
                    radius = read(parser,nameTag);
                    break;
                case "eventType":
                    eventType = read(parser,nameTag);
                    break;
                case "name":
                    name = read(parser,nameTag);
                    break;
                case "activated":
                    activated = read(parser,nameTag);
                    break;
                default:
                    skip(parser);
            }
        }

        if(null != activated
        && null != geoFenceId
        && null != latitude
        && null != longitude
        && null != radius
        && null != eventType
        && activated.equalsIgnoreCase("true")){
            return new GeoFenceVO(geoFenceId,latitude,longitude,radius,eventType,name);
        }else {
            return null;
        }
    }

    // Processes id tags in the feed.
    private static String read(XmlPullParser parser,String tag) throws IOException, XmlPullParserException {
        parser.require(XmlPullParser.START_TAG, ns, tag);
        String tagContent = readText(parser);
        parser.require(XmlPullParser.END_TAG, ns, tag);
        return tagContent;
    }

    // For the tags, extracts text values.
    private static String readText(XmlPullParser parser) throws IOException, XmlPullParserException {
        String result = "";
        if (parser.next() == XmlPullParser.TEXT) {
            result = parser.getText();
            parser.nextTag();
        }
        return result;
    }

    private static void skip(XmlPullParser parser) throws XmlPullParserException, IOException {
        if (parser.getEventType() != XmlPullParser.START_TAG) {
            throw new IllegalStateException();
        }
        int depth = 1;
        while (depth != 0) {
            switch (parser.next()) {
                case XmlPullParser.END_TAG:
                    depth--;
                    break;
                case XmlPullParser.START_TAG:
                    depth++;
                    break;
            }
        }
    }

}
