package com.ogangi.messangi.android.sdk.vo;

import android.util.Xml;

import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;

import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.List;

/**
 * Represents a Subscription retrieved from Messangi
 * providing access to subId, name and subscribed.
 */
public class SubscriptionVO {
	
	private String subId;
	private String name;
	private boolean subscribed;

    /**
     * Instantiates a new SubscriptionVO.
     *
     * @param subId the subscription ID
     * @param name the name of Subscription
     * @param subscribed true if user is Registered in Subscription, false otherwise
     */
    public SubscriptionVO(String subId, String name, boolean subscribed) {
		this.subId = subId;
		this.name = name;
		this.subscribed = subscribed;
	}

    /**
     * Gets the subscription ID.
     *
     * @return the subscriptionID
     */
    public String getSubId() {
		return subId;
	}

    /**
     * Set the subscription ID
     *
     * @param subId the subscription ID
     */
    public void setSubId(String subId) {
		this.subId = subId;
	}

    /**
     * Gets subscription name.
     *
     * @return the subscription name
     */
    public String getName() {
		return name;
	}

    /**
     * Sets subscription name.
     *
     * @param name the name of Subscription
     */
    public void setName(String name) {
		this.name = name;
	}

    /**
     * Sets if user is subscribed
     *
     * @param subscribed true if user is subscribed, false otherwise
     */
    public void setSubscribed(boolean subscribed) {
		this.subscribed = subscribed;
	}

    /**
     * Is subscribed.
     *
     * @return true if user is subscribed, false otherwise.
     */
    public boolean isSubscribed() {
		return subscribed;
	}


	//TODO: Considerar mover esto a un padre comun de todas las clases VO y extender
	private static final String ns = null;

	public static List parses(String in) throws XmlPullParserException, IOException {
		XmlPullParser parser = Xml.newPullParser();
		parser.setFeature(XmlPullParser.FEATURE_PROCESS_NAMESPACES, false);
		parser.setInput(new StringReader(in));
		parser.nextTag();
		return readFeed(parser);
	}

	private static List readFeed(XmlPullParser parser) throws XmlPullParserException, IOException {
		ArrayList entries = new ArrayList();

		parser.require(XmlPullParser.START_TAG, ns, "distributionLists");
		while (parser.next() != XmlPullParser.END_TAG) {
			if (parser.getEventType() != XmlPullParser.START_TAG) {
				continue;
			}
			String name = parser.getName();
			// Starts by looking for the entry tag
			if (name.equals("distributionlist")) {
				entries.add(parse(parser));
			} else {
				skip(parser);
			}
		}
		return entries;
	}

	// Parses the contents of an XML entry. If it encounters a successful or message tag, hands them off
	// to their respective "read" methods for processing. Otherwise, skips the tag.
	public static SubscriptionVO parse(XmlPullParser parser) throws XmlPullParserException, IOException {
		parser.require(XmlPullParser.START_TAG, ns, "distributionlist");
		String id = null;
		String name = null;
		boolean subscribed = false;

		while (parser.next() != XmlPullParser.END_TAG) {
			if (parser.getEventType() != XmlPullParser.START_TAG) {
				continue;
			}
			String nameTag = parser.getName();
			if (nameTag.equals("id")) {
				id = readId(parser);
			} else if (nameTag.equals("name")) {
				name = readName(parser);
			} else if (nameTag.equals("subscribed")){
				subscribed = readStatus(parser);
			}else {
				skip(parser);
			}
		}
		return new SubscriptionVO(id,name,subscribed);
	}

	// Processes id tags in the feed.
	private static String readId(XmlPullParser parser) throws IOException, XmlPullParserException {
		parser.require(XmlPullParser.START_TAG, ns, "id");
		String status = readText(parser);
		parser.require(XmlPullParser.END_TAG, ns, "id");
		return status;
	}

	// Processes name tags in the feed.
	private static String readName(XmlPullParser parser) throws IOException, XmlPullParserException {
		parser.require(XmlPullParser.START_TAG, ns, "name");
		String message = readText(parser);
		parser.require(XmlPullParser.END_TAG, ns, "name");
		return message;
	}

	// Processes subscribed tags in the feed.
	private static boolean readStatus(XmlPullParser parser) throws IOException, XmlPullParserException {
		parser.require(XmlPullParser.START_TAG, ns, "subscribed");
		String message = readText(parser);
		parser.require(XmlPullParser.END_TAG, ns, "subscribed");
		return !message.equals("no");
	}


	// For the tags, extracts text values.
	private static String readText(XmlPullParser parser) throws IOException, XmlPullParserException {
		String result = "";
		if (parser.next() == XmlPullParser.TEXT) {
			result = parser.getText();
			parser.nextTag();
		}
		return result;
	}

	private static void skip(XmlPullParser parser) throws XmlPullParserException, IOException {
		if (parser.getEventType() != XmlPullParser.START_TAG) {
			throw new IllegalStateException();
		}
		int depth = 1;
		while (depth != 0) {
			switch (parser.next()) {
				case XmlPullParser.END_TAG:
					depth--;
					break;
				case XmlPullParser.START_TAG:
					depth++;
					break;
			}
		}
	}

}

